
/*
 * 	K150 Program and Layer definitions
 * 	derived from the K150 Version 1.6 Software document
 *
 *	Mike Hall mghall@enteract.com
 */

/*
 *	NOTES: 	Bit-fields are not portable!
 *		They are used here for illustration only.
 *		Assume bits are assigned from bottom to top, low to high.
 *		The symbol names are too long for real use.
 */

#define	MAX_LAYERS	21	/* maximum number of layers in a program */

/*
 *	K150 program layout
 */
struct k150_program {
	struct	k150_phdr	prog;			/* program header */
	struct	k150_layer	layer[MAX_LAYERS];	/* one or more layers */
};

/*
 *	Layout of the program header.
 */
struct k150_phdr {
	long	nlayer_right	:3,	/* number of layers in right region */
		nlayer_center	:3,	/* number of layers in center region */
		nlayer_left	:3,	/* number of layers in left region */
		right_split	:7,	/* right split point, MIDI key number */
		left_split	:7,	/* left split point, MIDI key number */
		pad1		:1,	/* unused bit 23 */
		prog		:8;	/* program number */

	long	soft_pedal	:6,	/* soft pedal range, dB */
		vib_delay	:10,	/* vibrato onset delay, ms */
		pw_range	:7,	/* pitch-wheel range, cents */
		kp_range	:7,	/* key-pressure range, cents */
		pad2		:2,	/* unused bits 30/31 */

	long	vib_rate	:9,	/* vibrato rate, 0.1 Hz increments */
		vib_sym		:5,	/* vibrato symmetry, +/-N */
		vib_shape	:2,	/* vibrato shape: 0=tri, 1=square */
		vib_depth	:14,	/* vibrato depth, cents */
		vib_mode	:2;	/* vibrato mode */

	long	ch_delay	:12,	/* chorus delay, ms */
		ch_decay	:4,	/* chorus decay, dB */
		ch_detune	:11,	/* chorus detune, cents */
		ch_mode		:2,	/* chorus mode */
		ch_notes	:3;	/* chorus extra notes */

	long	eq62		:6,	/* equalization 62.5Hz, +/-dB */
		eq125		:6,	/* equalization 125Hz, +/-dB */
		eq250		:6,	/* equalization 250Hz, +/-dB */
		eq500		:6,	/* equalization 500Hz, +/-dB */
		left_bal	:5,	/* left region balance, +/-dB */
		pad3		:3;	/* unused bits 29/30/31 */

	long	eq1k		:6,	/* equalization 1kHz, +/-dB */
		eq2k		:6,	/* equalization 2kHz, +/-dB */
		eq4k		:6,	/* equalization 4kHz, +/-dB */
		eq8k		:6,	/* equalization 8kHz, +/-dB */
		right_bal	:5,	/* right region balance, +/-dB */
		pad4		:3;	/* unused bits 29/30/31 */
	
	char	prog_name[8];		/* program name, ASCII, space filled */
};

/* vibrato shape values */
enum VSHAPE {
	TRIANGLE = 0,
	SQUARE	 = 1
};

/* vibrato and chorus mode values */
enum VCDIR {
	ABOVE_BELOW 	= 0,
	ABOVE		= 1,
	BELOW		= 2,
	BELOW_ABOVE	= 3
};


/*
 *	Layout of the layer.
 */
struct k150_layer {
	long	trans		:7,	/* layer transpose, +/-semitones */
		t_shift,	:7,	/* timbre shift, +/-semitones */
		t_mode,		:2,	/* timbre mode */
		detune		:8,	/* layer detune, +/-cents */
		voicenum	:8;	/* voice number */

	long	t_level		:7,	/* timbre level, +/-dB */
		pad5		:9,	/* unused bits 7-15 */
		bend_mode	:2,	/* pitch bend mode */
		vib_mode	:2,	/* vibrato mode */
		ch_mode		:2,	/* chorus mode */
		bal_mode	:2,	/* balance mode */
		balance		:5,	/* layer balance, +/-dB */
		eqsw		:1;	/* equalization on/off switch */
};

/* values for timbre-mode */
enum TMODE {
	T_NORMAL = 0,		/* normal setting */
	T_SLIDER = 1,		/* timbre-shift controlled by slider */
	T_VELOC	 = 2,		/* timbre-shift controlled by velocity */
	T_SELECT = 3		/* timbre-shift is MIDI key # in 'v_shift' */
};

/* values for pitch-bend mode */
enum PBMODE {
	PB_OFF	 = 0,		/* pitch bend doesn't affect this layer  */
	PB_WHEEL = 1,		/* layer is bent with pitch-wheel */
	PB_PRESS = 2,		/* layer is bent with key pressure */
	PB_BOTH	 = 3,		/* layer is bent with p-wheel and k-press */
};

/* values for vibrato and chorus modes */
enum VCMODE {
	VC_OFF	 = 0,		/* effect is turned off in this layer */ 
	VC_WHEEL = 1,		/* depth is controlled by mod wheel */
	VC_PRESS = 2,		/* depth is controlled by key pressure */
	VC_FIXED = 3,		/* depth is set at fixed amount */
};

/* values for balance mode */
enum BMODE {
	B_FIXED	 = 0,		/* balance setting is static */
	B_SLIDER = 1,		/* balance is controlled by data slider */
	B_VELOC	 = 2,		/* balance is controlled by velocity */
};

/* values for equalizer switch */
enum EQSWITCH {
	EQ_ON	= 0,		/* eq is on for this layer */
	EQ_OFF	= 1,		/* eq is off for this layer */
				/* (that's what the diagram says) */
};


